; $Id: benchmark_1mon.pro,v 1.3 2008/05/07 20:32:19 bmy Exp $
;-----------------------------------------------------------------------
;+
; NAME:
;        BENCHMARK_1MON
;
; PURPOSE:
;        Produces maps of tracers and other quantities from two
;        GEOS-Chem benchmark simulations (for model validation).
;
; CATEGORY:
;        GEOS-Chem Benchmarking
;
; CALLING SEQUENCE:
;        BENCHMARK_1MON [, Keywords ]
;
; INPUTS:
;        None
;
; KEYWORD PARAMETERS:
;        DEBUG -> Set this switch to print the values of the various
;             input variables.  Use this to make sure that all values
;             have been created corectly.
;
;        /DO_CONC_MAPS -> Set this switch to include maps of tracer
;             concentration at the surface and 500hPa levels in the
;             set of generated plots.  Default is not to include
;             tracer concentration maps.
;
;        /DYNRANGE -> Set this switch to create additional difference 
;             plots, ratio plots, and profile plots using the whole
;             dynamic range of the data.
;
;        INPUTFILE -> A file containing default values for version 
;             numbers, directories, model names, resolutions, etc.
;             Default is "input_bm.1mon"
;
;        /NO_FREQ_DIST -> Set this switch to suppress printing of
;             the frequency distribution plot.  This is necessary
;             if you are comparing two models (e.g. GEOS-4 and
;             GEOS-5) with a different number of levels).
;
; OUTPUTS:
;        None
;
; SUBROUTINES:
;        External Subroutines Required:
;        ====================================================
;        DynOutFile (function)    GetSfc500Levels (function)  
;
;        External Subroutines Required:
;        ====================================================
;        CTM_NAMEXT (function)    CTM_TYPE (function) 
;        DIFFERENCES              FREQ_DIST         
;        FULLCHEM_BUDGET          FULLCHEM_EMISSIONS
;        JV_RATIO                 PROFILES
;        MAPS                     NYMD2TAU (function)
;        RATIOS                   REPLACE_TOKEN
;        STRUADDVAR (function)
;
; REQUIREMENTS:
;        References routines from the GAMAP package.
;
; NOTES:
;        (1) The following types of plots are produced:        
;
;            For Full Chemistry Benchmark Simulations:
;            =========================================
;            (a) J-value ratios (new/old) at surface and 500 hPa 
;            (b) Frequency distribution of tracer ratios (new/old)
;            (c) Tracer difference maps at surface and 500 hPa
;            (d) Tracer ratio maps at surface and 500 hPa
;            (e) Tracer profiles along 15S and 42N
;            (f) Budgets of Ox and CO and mean OH lifetime
;            (g) Emissions totals for "New" and "Old" versions
;
;        (2) Assumes that a series of diagnostics is included 
;            in the new and old GEOS-CHEM benchmark punch files:
;            (a) ND22 ("JV-MAP-$")   (h) ND44 ("DRYD-FLX")
;            (b) ND24 ("EW-FLX-$")   (i) ND45 ("IJ-AVG-$")
;            (c) ND25 ("NS-FLX-$")   (j) ND65 ("PORL-L=$")
;            (d) ND26 ("UP-FLX-$")   (k) ND66 ("DAO-3D-$")
;            (e) ND31 ("PS-PTOP" )   (l) ND67 ("DAO-FLDS")
;            (f) ND43 ("CHEM-L=$")   (m) ND69 ("DXYP"    )
;            (g) ND32 (""NOX-AC-$", "NOX-AN-$", "NOX-BIOB", 
;                       "NOX-FERT", "NOX-LI-$", "NOX-SOIL")
;
; EXAMPLES:
;        BENCHMARK_1MON, /DEBUG
;
;            ; Produces benchmark output from full chemistry
;            ; diagnostic output.  Will read input parameters
;            ; from "input_bm.1mon"
;
;        BENCHMARK_1MON, /DEBUG, /DYNRANGE
;
;            ; Same as above, but will also generate difference
;            ; and ratio plots using the full dynamic range of
;            ; the data in addition to the predetermined data
;            ; ranges.
;
;        BENCHMARK_1MON, /DEBUG, /DYNRANGE, /DO_CONC_MAPS
;
;            ; Same as above, but will generate maps of tracer
;            ; concentration along with the other types of plots.
;
; MODIFICATION HISTORY:
;        bmy, 09 Nov 2007: VERSION 1.01
;                          - based on "benchmark.pro"
;
;-
; Copyright (C) 2007, 
; Bob Yantosca and Philippe Le Sager, Harvard University
; This software is provided as is without any warranty whatsoever. 
; It may be freely used, copied or distributed for non-commercial 
; purposes. This copyright notice must be kept with any copy of 
; this software. If this software shall be used commercially or 
; sold as part of a larger package, please contact the author.
; Bugs and comments should be directed to bmy@io.as.harvard.edu
; or phs@io.harvard.edu with subject "IDL routine benchmark_1mon"
;-----------------------------------------------------------------------

function DynOutFile, File

   ;====================================================================
   ; Internal function DynOutFile appends the the extension 
   ; '.dyn_range' to a PostScript filename in order to denote
   ; that the filename contains the dynamic range of data, as opposed
   ; to the "small" range of data. (bmy, 11/14/07)
   ;====================================================================

   ; Add text and return
   return, StrMid( File, 0, StrLen( File )-3L ) + '.dyn_range.ps' 
end

;------------------------------------------------------------------------------

function GetSfc500Levels, ModelName

   ;====================================================================
   ; Internal function GetSfc500Level returns the surface and 500hPa 
   ; levels, depending on the met field type (bmy, 11/14/07)
   ;====================================================================

   ; Surface level
   Level_Sfc  = 1

   ; 500hPa level
   case ( ModelName ) of
      'geos5' : Level_500 = 23
      'geos4' : Level_500 = 8
      'geos3' : Level_500 = 13
      else    : Level_500 = -1
   endcase

   ; Return to calling program
   return, [ Level_Sfc, Level_500 ]
end

;-----------------------------------------------------------------------------

pro BenchMark_1Mon, InputFile,                                     $
                    Debug=Debug,        Do_Conc_Maps=Do_Conc_Maps, $
                    DynRange=DynRange,  No_Freq_Dist=No_Freq_Dist, $
                    _EXTRA=e
   
   ;====================================================================
   ; Initialization 
   ;====================================================================

   ; External Functions
   FORWARD_FUNCTION CTM_NamExt,    CTM_Type,  Nymd2Tau, $
                    Replace_Token, StruAddVar

   ; Keywords
   DynRange     = Keyword_Set( DynRange )
   Debug        = Keyword_Set( Debug )
   Do_Conc_Maps = Keyword_Set( Do_Conc_Maps ) 
   Do_Freq_Dist = 1L - Keyword_Set( No_Freq_Dist )
   if ( N_Elements( InputFile ) eq 0 ) then InputFile = 'input_bm.1mon'

   ; Read the default ranges for difference plots
   Read_Diff_Range, 'diff_range.1mon'
   
   ;====================================================================
   ; Read default settings from INPUTFILE
   ;====================================================================

   ; Open the input file
   Open_File, InputFile, Ilun, /Get_Lun

   ; Define string variable
   Line = ''
   
   ; Loop thru file
   while ( not EOF( Ilun ) ) do begin

      ; Read line and break on colon
      ReadF, Ilun, Line
      Result = StrTrim( StrBreak( Line, ':' ), 2 )

      ; Parse value of line into individual variables
      case ( StrUpCase( Result[0] ) ) of 
         'VERSION_1'   : Version_1   = Result[1]
         'VERSION_2'   : Version_2   = Result[1]
         'MODEL_1'     : Model_1     = CTM_NamExt( CTM_Type( Result[1] ) )
         'MODEL_2'     : Model_2     = CTM_NamExt( CTM_Type( Result[1] ) )
         'DATE_1'      : Tau0_1      = Nymd2Tau( Long( Result[1] ) )
         'DATE_2'      : Tau0_2      = Nymd2Tau( Long( Result[1] ) )
         'N_TRACERS_1' : N_Tracers_1 = Long( Result[1] )
         'N_TRACERS_2' : N_Tracers_2 = Long( Result[1] )
         'RUNDIR_1'    : RunDir_1    = Result[1]
         'RUNDIR_2'    : RunDir_2    = Result[1]
         'FILE_1'      : File_1      = Result[1]
         'FILE_2'      : File_2      = Result[1]
         'INITFILE_2'  : InitFile_2  = Result[1]
         'FINALFILE_2' : FinalFile_2 = Result[1]
         'OUTPUTDIR'   : OutPutDir   = Result[1]
         'MAX_ALT_KM'  : Max_Alt_Km  = Long( Result[1] )
         'FREQ_DIST'   : Freq_Dist   = Result[1]
         'DIFFERENCES' : Diff_Plots  = Result[1]
         'RATIOS'      : Ratio_Plots = Result[1]
         'PROFILES'    : Prof_Plots  = Result[1] 
         'J-VALUES'    : J_Values    = Result[1]
         'CONC_MAPS'   : Conc_Plots  = Result[1]
         'BUDGET'      : Budget      = Result[1] 
         'EMISSIONS'   : Emissions   = Result[1] 
         else          : ; Do nothing
      endcase
   endwhile

   ; Close input filename
   Close,    Ilun
   Free_Lun, Ilun

   ;----------------
   ; Error checks!
   ;----------------

   ; Skip error check if we are comparing two different model types
   if ( Do_Freq_Dist ) then begin

      ; Can't plot if the vertical grids are different!
      if ( Model_1 ne Model_2 ) then begin
         Message, 'The two models have different vertical grids!  Cannot plot!'
      endif

       ; Can't plot if the # of tracers are different!
      if ( N_Tracers_1 ne N_Tracers_2 ) then begin
         Message, 'The two models have a different # of tracers!  Cannot plot!'
      endif

   endif

   ;====================================================================
   ; Create directory, filename, and other relevant variables
   ;====================================================================

   ; Define structure for token replacement
   Token       = { VERSION_1:Version_1,  VERSION_2:Version_2, $
                   MODEL_1  :Model_1,    MODEL_2  :Model_2   }

   ; Replace tokens in run directory variables
   RunDir_1    = Replace_Token( RunDir_1, Token )
   RunDir_2    = Replace_Token( RunDir_2, Token )

   ; Add run directory variables to TOKEN structure
   Token       = StruAddVar( Token, { RUNDIR_1:RunDir_1, RUNDIR_2:RunDir_2 } )

   ; Replace tokens in output directory variable
   OutputDir   = Replace_Token( OutputDir,   Token )

   ; Add output directory to TOKEN structure
   Token       = StruAddVar( Token, { OUTPUTDIR:OutputDir } )

   ; Replace tokens in the rest of the variables
   File_1      = Replace_Token( File_1,      Token ) 
   File_2      = Replace_Token( File_2,      Token ) 
   InitFile_2  = Replace_Token( InitFile_2,  Token ) 
   FinalFile_2 = Replace_Token( FinalFile_2, Token ) 
   Freq_Dist   = Replace_Token( Freq_Dist,   Token )
   Diff_Plots  = Replace_Token( Diff_Plots,  Token )
   Ratio_Plots = Replace_Token( Ratio_Plots, Token )
   Prof_Plots  = Replace_Token( Prof_Plots,  Token )
   Conc_Plots  = Replace_Token( Conc_Plots,  Token )
   J_Values    = Replace_Token( J_Values,    Token )
   Budget      = Replace_Token( Budget,      Token )
   Emissions   = Replace_Token( Emissions,   Token )

   ; Altitude range (km) for difference profile plots
   AltRange    = [ 0L, Max_Alt_Km ]

   ; Surface and 500hPa levels from both models ( sfc1, sfc2, 500_1, 500_2 )
   Levels_1    = GetSfc500Levels( Model_1 )
   Levels_2    = GetSfc500Levels( Model_2 )
   Levels      = [ Levels_1[0], Levels_2[0], Levels_1[1], Levels_2[1] ]

   ; TAU0 values
   Taus        = [ Tau0_1, Tau0_2 ]

   ; File names
   Files       = [ File_1, File_2 ]

   ; Tracer list
   Tracers     = LindGen( N_Tracers_1 ) + 1L

   ; Model versions
   Versions    = [ Version_1, Version_2 ]

   ;====================================================================
   ; Debug output
   ;====================================================================
   if ( Debug ) then begin

      print, '%%% 1st Model %%%'
      print, 'Version_1   : ', Version_1
      print, 'Model_1     : ', Model_1     
      print, 'Tau0_1      : ', Tau0_1       
      print, 'N_Tracers_1 : ', N_Tracers_1
      print, 'RunDir_1    : ', RunDir_1    
      print, 'File_1      : ', File_1      
      print
      print, '%%% 2nd Model %%%'      
      print, 'Version_2   : ', Version_2
      print, 'Model_2     : ', Model_2     
      print, 'Tau0_2      : ', Tau0_2       
      print, 'N_Tracers_2 : ', N_Tracers_2 
      print, 'RunDir_2    : ', RunDir_2    
      print, 'File_2      : ', File_2      
      print, 'InitFile_2  : ', InitFile_2  
      print, 'FinalFile_2 : ', FinalFile_2
      print
      print, '%%% For Plotting %%%'
      print, 'OutputDir   : ', OutputDir   
      print, 'Altrange    : ', Altrange
      print, 'Freq_Dist   : ', Freq_Dist  
      print, 'Diff Plots  : ', Diff_Plots
      print, 'Ratio Plots : ', Ratio_Plots 
      print, 'Prof_Plots  : ', Prof_Plots
      print, 'Conc_Plots  : ', Conc_Plots
      print, 'J_Values    : ', J_Values   
      print, 'Budget      : ', Budget  
      print, 'Emissions   : ', Emissions  

   endif

   ;====================================================================
   ; Make the plots
   ;====================================================================

   ;--------------------------------
   ; Frequency distribution
   ;--------------------------------

   ; NOTE: sometimes it is necessary to skip the frequency distribution 
   ; plot, especially if the 2 models have a different # of levels
   ; (i.e. the first time you are testing a new met field type, etc.)
   if ( Do_Freq_Dist ) then begin

      ; Echo info
      Message, 'Generating frequency distributions ...', /Info

      ; Create frequency distribution plot
      Freq_Dist, Files, Taus, Tracers, Versions, /PS, OutFile=Freq_Dist, $
                 /LVarTrop

   endif

   ;---------------------------------
   ; J-value ratios @ sfc & 500hPa
   ;---------------------------------

   ; Echo info
   Message, 'Generating J-value ratio maps ...', /Info

   ; Create J-value plots with "small" data range
   JV_Ratio, Files, Levels, Taus, Versions, /PS, OutFile=J_Values

   ; Create J-value plots with full dynamic range (if necessary)
   if ( DynRange ) then begin
      JV_Ratio, Files, Levels, Taus, Versions, $
         /DynRange, /PS, OutFile=DynOutFile( J_Values )
   endif

   ;---------------------------------
   ; Tracer ratios @ sfc & 500hPa
   ;---------------------------------
   
   ; Echo info
   Message, 'Generating tracer ratio maps ...', /Info
    
   ; Create ratio plots with "small" data range
   Ratios, Files, Levels, Taus, Tracers, Versions, /PS, OutFile=Ratio_Plots

   ; Create ratio plots with full dynamic range (if necessary)
   if ( DynRange ) then begin
      Ratios, Files, Levels, Taus, Tracers, Versions, $
         /DynRange, /PS, OutFile=DynOutFile( Ratio_Plots )
   endif

   ;---------------------------------
   ; Tracer abs diff @ sfc & 500hPa
   ;---------------------------------

   ; Echo info
   Message, 'Generating tracer difference maps ...', /Info
   
   ; Create diff plots with "small" data range
   Differences, Files, Levels, Taus, Tracers, Versions, /PS, OutFile=Diff_Plots

   ; Create diff plots with full dynamic range (if necessary)
   if ( DynRange ) then begin
      Differences, Files, Levels, Taus, Tracers, Versions, $
         /DynRange, /PS, OutFile=DynOutFile( Diff_Plots )
   endif

   ;---------------------------------
   ; Tracer profiles @ sfc & 500hPa
   ;---------------------------------

   ; NOTE: sometimes it is necessary to skip the frequency distribution 
   ; plot, especially if the 2 models have a different # of levels
   ; (i.e. the first time you are testing a new met field type, etc.)
   if ( Do_Freq_Dist ) then begin


      ; Echo info
      Message, 'Generating difference profiles along 15S and 42N ...', /Info
   
      ; Create lon diff profile plots w/ "small" data range
      Profiles, Files, AltRange, Taus, Tracers, Versions, $
                /PS, OutFile=Prof_Plots

      ; Create diff plots with full dynamic range (if necessary)
      if ( DynRange ) then begin
         Profiles, Files, AltRange, Taus, Tracers, Versions, $
                   /DynRange, /PS, OutFile=DynOutFile( Prof_Plots )
      endif

   endif

   ;---------------------------------
   ; Tracer concentration maps 
   ;---------------------------------
   if ( Do_Conc_Maps ) then begin

      ; Echo info
      Message, 'Generating tracer concentration maps ...', /Info
   
      ; Create lon diff profile plots w/ "small" data range
      Maps, Files[1], Levels, Taus[1], Tracers, Versions[1], $
         /PS, OutFile=Conc_Plots

   endif

   ;---------------------------------
   ; Emission totals printout
   ;---------------------------------

   Message, 'Generating emission totals ...', /Info
      
   FullChem_Emissions, FileNew=File_2,    FileOld=File_1, $
                       OutFile=Emissions, _EXTRA=e
   
   ;#### KLUDGE, return w/o printing budgets (bmy, 11/20/08)
   return

   ;---------------------------------
   ; Budget printout
   ;---------------------------------

   Message, 'Generating budgets ...', /Info

   FullChem_Budget, Tau0=Taus[1],           FileNew=File_2,        $
                    InitialFile=InitFile_2, FinalFile=FinalFile_2, $
                    OutFile=Budget,         _EXTRA=e

end
